#include "../../include/v_lib.hpp"
#include  <stdio.h>
#include  <windows.h>
#include  <gl/gl.h>
#include  <gl/glu.h>
#include  <gl/glaux.h>
#include  <float.h>
#include  <math.h>

#define   WIN_SIZEX 512
#define   WIN_SIZEY 512
#define   MAX_TRIANGLES         512

static    int numtriangles=0;
scalar_t  scale = 0.1;
static    bytergba_t colors[MAX_TRIANGLES];
static    vec3_t triangles[MAX_TRIANGLES*3];

extern "C"
{
  void      VEXPORT draw_setscale(scalar_t scale);
  void      VEXPORT draw_setcolor(byte r,byte g,byte b);
  void      VEXPORT draw_addtriangle(vec3_t a,vec3_t b,vec3_t c);
  void      VEXPORT draw_main();
};

void VEXPORT draw_setscale(scalar_t scale) {::scale=scale;}

void VEXPORT draw_setcolor(byte r,byte g,byte b) 
{
  colors[numtriangles][0]=r;
  colors[numtriangles][1]=g;
  colors[numtriangles][2]=b;
  colors[numtriangles][3]=128;
}

void VEXPORT draw_addtriangle(vec3_t a,vec3_t b,vec3_t c)
{
  vec3Copy(triangles[numtriangles*3],a);
  vec3Copy(triangles[numtriangles*3+1],b);
  vec3Copy(triangles[numtriangles*3+2],c);
  numtriangles++;
}

float alpha = 0, betta = 0;

void drawscene(int milliseconds)
{
  int i;
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  
  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);
  glRotatef(alpha,1,0,0);
  glRotatef(betta,0,0,1);
  glScalef(scale,scale,scale);
  glEnable(GL_BLEND);
  glBlendFunc(GL_ONE,GL_SRC_COLOR);
  glBegin(GL_TRIANGLES);
    for(i=0;i<numtriangles;i++)
    {
      glColor4ubv((byte*)(&colors[i]));
      glVertex3fv(triangles[i*3]);
      glVertex3fv(triangles[i*3+1]);
      glVertex3fv(triangles[i*3+2]);
    }
  glEnd();

  alpha += (float)milliseconds / 200;
  betta += (float)milliseconds / 400;
  glPopMatrix();
}

void CALLBACK Resize(GLsizei width, GLsizei height )
{
  GLfloat aspect;
  if(!width || !height) return;
  aspect=(float)width/(float)height;
  glViewport(0,0,width,height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-aspect,aspect,-1,1,-1,1);
  glMatrixMode(GL_MODELVIEW);
}    

int PrevTick;

void CALLBACK Draw()
{
  int Tick;
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  Tick=GetTickCount();
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
  glDisable(GL_CULL_FACE);
  drawscene(Tick - PrevTick);
  PrevTick=Tick;
  glFlush();
  auxSwapBuffers();
}

void VEXPORT draw_main()
{
  auxInitPosition(20,20,WIN_SIZEX,WIN_SIZEY);
  auxInitDisplayMode(AUX_RGB|AUX_DOUBLE);
  if(!auxInitWindow("Pyramid")) auxQuit();
  auxReshapeFunc(Resize);
  SetClassLong(GetForegroundWindow(),GCL_HBRBACKGROUND,0);
  PrevTick=GetTickCount();
  auxIdleFunc(Draw);
  Resize(WIN_SIZEX,WIN_SIZEY);
  auxMainLoop(Draw);
}
