#include "../../include/v_lib.hpp"
#include "../../devel/include/v_show.hpp"
#pragma hdrstop
#include <stdio.h>

struct pyramid_t
{
  vec3_t vertices[4];
  void set(int vert, scalar_t x, scalar_t y, scalar_t z)
  {
    VASSERT(vert<4);
    vertices[vert][0] = x;
    vertices[vert][1] = y;
    vertices[vert][2] = z;
  }
  void display( void )
  {
    puts( "=================================================" );
    printf( "p: A(%g,%g,%g); B(%g,%g,%g); C(%g,%g,%g); "
            "D(%g,%g,%g).\n", vertices[0][0], vertices[0][1], vertices[0][2],
                              vertices[1][0], vertices[1][1], vertices[1][2],
                              vertices[2][0], vertices[2][1], vertices[2][2],
                              vertices[3][0], vertices[3][1], vertices[3][2] );
  }
};

typedef struct solution_s
{
  vec3_t AB, AC, AD;          //  - ॡ
  scalar_t aAB, aAC, aAD;     //  㫨
  scalar_t ABmAC;             // ᪠୮ ந AB  AC
  scalar_t pracAB, prabAC;    // ஥樨 AB  AC  AC  AB
  scalar_t cosa;              // ᨭ 㣫  AB  AC
  scalar_t Sabc;              // 頤 ᭮
  vec3_t ABcAC, ABcAD;        //  ந
  scalar_t mixABACAD, mixACABAD;  // ᬥ蠭 ந
  scalar_t volume;            // ꥬ
  scalar_t aDDs;              //  ᭮
  plane3_t pABC;              // ᪮ ᭮
  line3_t lDDs;               // ࠬ᪮ ࠢ ᭮
  vec3_t Ds;                  // ஥ 쥩 設  ᭮
  vec3_t DDs;                 //   設  ஥樨
} solution_t;

void solve_task(solution_t *sol,pyramid_t *dat)
{
  //  塞 p p
  vec3Sub(sol->AB,dat->vertices[1],dat->vertices[0]);
  vec3Sub(sol->AC,dat->vertices[2],dat->vertices[0]);
  vec3Sub(sol->AD,dat->vertices[3],dat->vertices[0]);
  //  塞 ᬥ蠭 p
  sol->mixABACAD = vec3Mix(sol->AB,sol->AC,sol->AD);
  VASSERT(IsZero(sol->mixABACAD)); // zero volumey
  sol->mixACABAD = - sol->mixABACAD;
  //  塞 㫨 p
  sol->aAB = vec3Abs(sol->AB);
  sol->aAC = vec3Abs(sol->AC);
  sol->aAD = vec3Abs(sol->AD);
  //  塞 ᪠p p, p樨, ᨭ 㣫
  sol->ABmAC = vec3Dot(sol->AB,sol->AC);
  sol->pracAB = sol->ABmAC / sol->aAC;
  sol->prabAC = sol->ABmAC / sol->aAB;
  sol->cosa = sol->prabAC / sol->aAC;
  //  塞 p p
  vec3Cross(sol->ABcAC,sol->AB,sol->AC);
  vec3Cross(sol->ABcAD,sol->AB,sol->AD);
  //  塞 頤 ᭮
  sol->Sabc = vec3Abs(sol->ABcAC)/2;
  //  塞 ꥬ
  sol->volume = v_abs(sol->mixABACAD) / 6;
  //  塞  ᭮
  sol->aDDs = 3 * sol->volume / sol->Sabc;
  //  塞 樥 p ᪮
  plane3FromPoints(&sol->pABC,dat->vertices[0],dat->vertices[1],dat->vertices[2]);
  //  塞 樥 p 
  line3FromDirAndOrigin(&sol->lDDs,dat->vertices[3],sol->pABC.normal);
  //  室  p祭   ᭮
  vec3FromLineAndPlane(sol->Ds,&sol->lDDs,&sol->pABC);
  //  室 p p D -> Ds
  vec3Copy(sol->DDs,sol->Ds);
  vec3Sub(sol->DDs,sol->DDs,dat->vertices[3]);
  VASSERT(!IsZero(vec3Abs(sol->DDs)-sol->aDDs));
}

void display_solution(solution_t *sol)
{
  puts( "-------------------------------------------------" );
  printf( "p p: "
          "AB=(%g,%g,%g); AC=(%g,%g,%g); AD=(%g,%g,%g).\n",
          sol->AB[0], sol->AB[1], sol->AB[2], 
          sol->AC[0], sol->AC[1], sol->AC[2],
          sol->AD[0], sol->AD[1], sol->AD[2] );
  printf( "㫨 p:     |AB|=%g; |AC|=%g; |AD|=%g.\n",
          sol->aAB, sol->aAC, sol->aAD );
  printf( "p p p AB  AC: %g\n", sol->ABmAC );
  printf( "ᨭ 㣫  AB  AC: %g\n", sol->cosa );
  printf( "p p AB  p AC: %g\n", sol->pracAB );
  printf( "p p AC  p AB: %g\n", sol->prabAC );
  printf( "頤 ᭮ ABC:  %g\n", sol->Sabc );
  printf( "p p: "
          "[AB,AC]=(%g,%g,%g); [AB,AD]=(%g,%g,%g).\n",
          sol->ABcAC[0], sol->ABcAC[1], sol->ABcAC[2], 
          sol->ABcAD[0], sol->ABcAD[1], sol->ABcAD[2] );
  printf( "蠭 p: %g\n", sol->mixACABAD );
  printf( "ꥬ p:         %g\n", sol->volume );
  printf( " , 饭  D: %g\n", sol->aDDs );
  printf( "p p樨 D  ABC: (%g,%g,%g)\n",
          sol->Ds[0], sol->Ds[1], sol->Ds[2] );
  printf( "p p DD1:       (%g,%g,%g)\n",
          sol->DDs[0], sol->DDs[1], sol->DDs[2] );
  printf( "p ᪮ ABC:      (%g)x+(%g)y+(%g)z+(%g)=0\n",
           sol->pABC.normal[0], sol->pABC.normal[0], sol->pABC.normal[0], sol->pABC.dist );
  printf( "p , 饭  D:\n"
          " | x = (%g) + (%g)t\n"
          "<  y = (%g) + (%g)t\n"
          " | z = (%g) + (%g)t\n",
          sol->lDDs.origin[0], sol->lDDs.dir[0],
          sol->lDDs.origin[1], sol->lDDs.dir[1],
          sol->lDDs.origin[2], sol->lDDs.dir[2] );
  puts( "-------------------------------------------------" );
}

extern "C"
{
  void      VEXPORT draw_setscale(scalar_t scale);
  void      VEXPORT draw_setcolor(byte r,byte g,byte b);
  void      VEXPORT draw_addtriangle(vec3_t a,vec3_t b,vec3_t c);
  void      VEXPORT draw_main();
};

void main()
{
  pyramid_t oneta;
  oneta.set( 0,-2,-4,-2 );
  oneta.set( 1,-2, 4, 4 );
  oneta.set( 2,-2, 4,-2 );
  oneta.set( 3, 2, 2, 2 );
  oneta.display();
  solution_t sol;
  solve_task(&sol,&oneta);
  display_solution(&sol);
  draw_setscale(0.2);
  draw_setcolor(128,128,128);
  draw_addtriangle(oneta.vertices[0],oneta.vertices[2],oneta.vertices[1]);
  draw_setcolor(255,100,100);
  draw_addtriangle(oneta.vertices[0],oneta.vertices[1],oneta.vertices[3]);
  draw_setcolor(100,255,100);
  draw_addtriangle(oneta.vertices[1],oneta.vertices[2],oneta.vertices[3]);
  draw_setcolor(100,100,255);
  draw_addtriangle(oneta.vertices[2],oneta.vertices[0],oneta.vertices[3]);
  draw_main();
}

