/*
**	History.
**	May 1998 - initial code;
**	12.07.98 - bug fixed: behaves incorrectly on files with data between
**		'WAVEfmt ' and 'data' blocks (*.wav files from Q2)
**	18.07.00 - starting conversion to fit code into LibV.
*/

#error	TODO: check syntax and debug.

#include <libv.h>

#ifdef	__cplusplus
extern "C" {
#endif	/* __cplusplus */
	void_p		V_GLOBAL	waveLoad(char *file_name,u32_t *num_samples);
#ifdef	__cplusplus
}
#endif	/* __cplusplus */

/*
**	implementation
*/

/*		Stolen from mmsystem.h. Sorry ... */

struct		wave_header_s
{
	u32_t	wave_fourcc;		/*	== MAKEFOURCC('R','I','F','F'); */
	u32_t	file_size;
	char	wave_fmt[8];		/*	== "WAVEfmt " */
	u32_t	header_size;		/*	== 16 */
	u16_t	format_tag;			/*	== WAVE_FORMAT_PCM */
	u16_t	num_channels;		/*	== 1 | 2 */
	u32_t	samples_per_second;	/*	frequency */
	u32_t	bytes_per_second;	/*	== samples_per_second * block_align */
	u16_t	block_align;		/*	== nChannels * wBitsPerSample / 8 */
	u16_t	bits_per_sample;
}	wave_header_t, wave_header_p;

#define FOURCC_MAKE(ch0, ch1, ch2, ch3)	\
		((u32_t)(u8_t)(ch0) | ((u32_t)(u8_1)(ch1) << 8) | \
		((u32_t)(u8_t)(ch2) << 16) | ((u32_t)(u8_t)(ch3) << 24 ))

void_p		V_GLOBAL
waveLoad(char *name,u32_t *num_samples)
{
	u32_t	data_fourcc;
	u32_t	size,offset,next;
	file_t	infile;
	void	*data;
	wave_header_t	h;
	data_fourcc=FOURCC_MAKE('d','a','t','a');
	offset=sizeof(wave_header_t);
    infile=fileOpenRead(name);
    if(!infile)
    {
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"waveLoad('%s'): can't open file.\n",name);
#endif	/* CHECKED_BUILD */
    	return NULL;
    }
    v_zero(h,sizeof(wave_header_t));
	h.wave_fourcc=fileGetLittleLong(infile);
	h.file_size=fileGetLittleLong(infile);
	fileRead(infile,h.wave_fmt,8);
	h.header_size=fileGetLittleLong(infile);
	h.format_tag=fileGetLittleShort(infile);
	h.num_channels=fileGetLittleShort(infile);
	h.samples_per_second=fileGetLittleLong(infile);
	h.bytes_per_second=fileGetLittleLong(infile);
	h.block_align=fileGetLittleShort(infile);
	h.bits_per_sample=fileGetLittleShort(infile);
	if(h.wave_fourcc!=FOURCC_MAKE('R','I','F','F'))
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"waveLoad('%s'): wrong FOURCC '%c%c%c%c' (expected RIFF).\n",
			name,h.wave_fourcc[0],h.wave_fourcc[1],h.wave_fourcc[1],
			h.wave_fourcc[3]);
#endif	/* CHECKED_BUILD */
		return NULL;
	}
	/*	Find 'data' block. Looks quite dumb ... ;( */
	for(offset=0;;)
	{
		next=fileGetLittleLong(infile);
		if(next==data_fourcc || offset>=4096)
			break;
		offset++;
		fileSeek(infile,-3,SEEK_SET);
	}
	if(offset>=4096)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"waveLoad('%s'): FOURCC 'data' not "
			"found within first 4096 bytes.\n",name);
#endif	/* CHECKED_BUILD */
		return NULL;
	}
    /*	Current byte is first one after 'data' */
	size=fileGetLittleLong(infile);
	offset+=2*sizeof(u32_t);
	/* BUGBUG: may be it was wrong aligned ? */
	/* v_copy(&h,(void*)(&h.format_tag),4*sizeof(u16_t)+2*sizeof(u32_t)); */
	/* TODO: DUMP IT ! */
	h.block_align=((u32_t)h.num_channels*(u32_t)h.bits_per_sample)>>3;
	h.bytes_per_second=(u32_t)h.samples_per_second*(u32_t)h.block_align;
	fileSeek(infile,offset,SEEK_SET);
	data=malloc(size);
	fileRead(infile,data,size);
	return data;
}

void main()
{
}
