/*<html><body bgcolor=#000080><font color=#C0C0B0><strong><pre>*/
#include	"../include/v_lib.h"
#include	"../source/v_mt.h"

/*
 *	matAlloc
 */
matrix_p V_GLOBAL
matAlloc(u32_t rows,u32_t cols)
{
	matrix_t *m;
	u32_t i,size;
	V_ASSERT(rows>0 && cols>0);
	size=
		sizeof(matrix_t)+
		rows*sizeof(mvec_t*)+
		rows*cols*sizeof(mvec_t);
	m=(matrix_t*)malloc(size);
	V_ASSERT(m);
	m->rows=rows;
	m->cols=cols;
	m->cell=(mvec_t**)(m+1);
	m->data=(mvec_t*)(m->cell+rows);
	m->cell[0]=m->data;
	for(i=1;i<rows;i++)
		m->cell[i]=m->cell[i-1]+cols;
	return m;
}

/*
 *	matCopy
 */
void V_GLOBAL
matCopy(matrix_t *dest,matrix_t *src)
{
	u32_t i;
	V_ASSERT(src && dst);
	V_ASSERT(dest->rows==src->rows && dest->cols==src->cols);
	dest->cell[0]=dest->data;
	for(i=1;i<dest->rows;i++)
		dest->cell[i]=dest->cell[i-1]+dest->cols;
	for(i=0;i<src->rows;i++)
		v_copydw(dest->cell[i],src->cell[i],(src->cols*sizeof(mvec_t))>>2);
}

/*
 *	matDuplicate
 */
matrix_p V_GLOBAL
matDuplicate(matrix_t *ref)
{
	u32_t i;
	matrix_t *d;
	V_ASSERT(ref);
	d=matAlloc(ref->rows,ref->cols);
	/* We can't copy all block because pou32_ters in ref may be swapped */
	for(i=0;i<ref->rows;i++)
		v_copydw(d->cell[i],ref->cell[i],(ref->cols*sizeof(mvec_t))>>2);
	return d;
}

/*
 *	matCopyRow
 */
void	V_GLOBAL
matCopyRow(vector_t *dst,matrix_t *m,u32_t row)
{
	V_ASSERT(m);
	V_ASSERT(row>=0 && row<m->rows);
	V_ASSERT(dst->size == m->cols);
	v_copydw(dst->elements,m->cell[row],m->cols*(sizeof(mvec_t)>>2));
}


/*
 *	matCopyCol
 */
void	V_GLOBAL
matCopyCol(vector_t *dst,matrix_t *m,u32_t col)
{
	u32_t r;
	V_ASSERT(m);
	V_ASSERT(col>=0 && col<m->cols);
	V_ASSERT(dst->size == m->rows);
	for(r=0;r<m->rows;r++)
		dst->elements[r]=m->cell[r][col];
}

/*
 *	matDuplicateRow
 */
vector_p	V_GLOBAL
matDuplicateRow(matrix_t *m,u32_t row)
{
	vector_p dst;
	dst=vecAlloc(m->cols);
	matCopyRow(dst,m,row);
	return dst;
}

/*
 *	matDuplicateCol
 */
vector_p	V_GLOBAL
matDuplicateCol(matrix_t *m,u32_t col)
{
	vector_t *dst;
	dst=vecAlloc(m->rows);
	matCopyCol(dst,m,col);
	return dst;
}

/*
 *	matUnit
 */
void	V_GLOBAL
matUnit(matrix_t *m)
{
	u32_t i,min;
	V_ASSERT(m);
	matZero(m);
	min=(m->rows > m->cols) ? m->cols : m->rows;
	for(i=0;i<min;i++)
		m->cell[i][i]=1.0;
}

/*
 *	matAdd
 */
void	V_GLOBAL
matAdd(matrix_t *dest,matrix_t *m1,matrix_t *m2)
{
	u32_t i,j;
	V_ASSERT(dest && m1 && m2);
	V_ASSERT(m1->rows==m2->rows && m1->cols==m2->cols);
	V_ASSERT(dest->rows == m1->rows && dest->cols == m1->cols);
	for(i=0;i<dest->rows;i++)
		for(j=0;j<dest->cols;j++)
			dest->cell[i][j]=m1->cell[i][j]+m2->cell[i][j];
}

/*
 *	matSub
 */
void	V_GLOBAL
matSub(matrix_t *dest,matrix_t *m1,matrix_t *m2)
{
	u32_t i,j;
	V_ASSERT(dest && m1 && m2);
	V_ASSERT(m1->rows==m2->rows && m1->cols==m2->cols);
	V_ASSERT(dest->rows == m1->rows && dest->cols == m1->cols);
	for(i=0;i<dest->rows;i++)
		for(j=0;j<dest->cols;j++)
			dest->cell[i][j]=m1->cell[i][j]-m2->cell[i][j];
}

/*
 *	matNegate
 */
void	V_GLOBAL
matNegate(matrix_t *dest,matrix_t *m)
{
	u32_t i,j;
	V_ASSERT(dest && m);
	V_ASSERT(dest->rows == m->rows && dest->cols == m->cols);
	for(i=0;i<dest->rows;i++)
		for(j=0;j<dest->cols;j++)
			dest->cell[i][j]=-m->cell[i][j];
}

/*
 *	matCompare
 */
vboolean	V_GLOBAL
matCompare(matrix_t *m1,matrix_t *m2)
{
	u32_t i,j;
	V_ASSERT(m1 && m2);
	V_ASSERT(m1->rows==m2->rows && m1->cols==m2->cols);
	for(i=0;i<m1->rows;i++)
		for(j=0;j<m1->cols;j++)
			if(!IsZero(m1->cell[i][j]-m2->cell[i][j]))
				return false;
	return true;
}

/*
 *	matMin
 */
void	V_GLOBAL
matMin(matrix_t *m,u32_t *row,u32_t *col)
{
	u32_t i,j,r,c;
	V_ASSERT(m);
	r=c=0;
	for(i=0;i<m->rows;i++)
		for(j=0;j<m->cols;j++)
			if(m->cell[i][j]<m->cell[r][c]) 
			{
				r=i;
				c=j;
			}
	if(row)
		*row=r;
	if(col)
		*col=c;
}

/*
 *	matMax
 */
void	V_GLOBAL
matMax(matrix_t *m,u32_t *row,u32_t *col)
{
	u32_t i,j,r,c;
	V_ASSERT(m);
	r=c=0;
	for(i=0;i<m->rows;i++)
		for(j=0;j<m->cols;j++)
			if(m->cell[i][j]>m->cell[r][c]) 
			{
				r=i;
				c=j;
			}
	if(row)
		*row=r;
	if(col)
		*col=c;
}

/*
 *	matScale
 */
void	V_GLOBAL
matScale(matrix_t *dest,matrix_t *m,mvec_t x)
{
	u32_t r,c;
	V_ASSERT(dest && m);
	V_ASSERT(dest->rows == m->rows && dest->cols == m->cols);
	for(r=0;r<dest->rows;r++)
		for(c=0;c<dest->cols;c++)
			dest->cell[r][c]=m->cell[r][c]*x;
}

/*
 *	matVecMul
 */
void	V_GLOBAL
matVecMul(vector_t *dest,matrix_t *m,vector_t *v)
{
	u32_t i,j;
	V_ASSERT(m && v);
	V_ASSERT(dest->size==m->rows && m->cols==v->size);
	for(i=0;i<m->rows;i++)
	{
		dest->elements[i]=0.0;
		for(j=0;j<m->cols;j++)
			dest->elements[i]+=v->elements[j] * m->cell[i][j];
	}
}

/*
 *	matMatMul
 */
void	V_GLOBAL
matMatMul(matrix_t *dest,matrix_t *ml,matrix_t *mr)
{
	u32_t M,N,P;
	u32_t i,j,k;
	mvec_t s;
	V_ASSERT(dest && ml && mr);
	V_ASSERT(ml->cols==mr->rows && dest->rows==ml->rows && 
		dest->cols==mr->cols);
	M=ml->rows;
	N=ml->cols;
	P=mr->cols;
	for(i=0;i<M;i++)
	{
		for(j=0;j<P;j++)
		{
			s=0;
			for(k=0;k<N;k++)
				s+=ml->cell[i][k]*mr->cell[k][j];
			dest->cell[i][j]=s;
		}
	}
}

/*
 *	matTranspose
 */
void	V_GLOBAL
matTranspose(matrix_t *dest,matrix_t *m)
{
	u32_t r,c;
	V_ASSERT(m);
	V_ASSERT(dest->cols!=m->rows || m->cols!=dest->rows);
	for(c=0;c<m->cols;c++)
		for(r=0;m->rows;r++)
			dest->cell[c][r]=m->cell[r][c];
}

/*
 *	matDeterm
 */
mvec_t	V_GLOBAL
matDeterm(matrix_t *m,vboolean keepon)
{
	u32_t i,j,k,ind,N;
	mvec_t lead, elem, max, *row, res, **Rows;
	matrix_t *m1;
	V_ASSERT(m);
	V_ASSERT(m->rows==m->cols);
	if(keepon)
		m1=matDuplicate(m);
	else
		m1=m;
	Rows=m1->cell;
	N=m1->rows;
	res=1;
	for(k=0;k<N;k++)
	{
		/* find pivot element under diagonal in column #k */
		ind=k; max=v_abs(Rows[k][k]);
		for(i=k+1;i<N;i++)
		{
			elem=v_abs(Rows[i][k]);
			if(max<elem)
			{
				max=elem;
				ind=i;
			}
		}
		if(IsZero(max)) 
		{
			free(m1);
			return 0.0;
		}
		/* swap rows */
		if(k!=ind)
		{
			row=Rows[ind];
			Rows[ind]=Rows[k];
			Rows[k]=row;
			res=-res;
		}
		/* next iteration of Haussian elimination method */
		lead=Rows[k][k];
		res*=lead;
		for(j=k+1;j<N;j++)
			Rows[k][j]/=lead;
		for(i=k+1;i<N;i++)
			for(j=k+1;j<N;j++)
				Rows[i][j]-=Rows[i][k]*Rows[k][j];
	}
	free(m1);
	return res;
}

/*
 *	matSolve
 */
vector_p	V_GLOBAL
matSolve(matrix_t *a,vector_t *_b,vboolean keepon)
{
	u32_t i,j,k,ind;
	mvec_t elem,max,b,*row;
	mvec_t **Rows;
	mvec_t *B;
	u32_t N;
	matrix_p a1;
	vector_p b1;
	V_ASSERT(a && _b);
	V_ASSERT(a->rows==a->cols);
	V_ASSERT(a->rows==_b->size);
	if(keepon)
	{
		a1=matDuplicate(a);
		b1=vecDuplicate(_b);
	}
	else
	{
		a1=a;
		b1=_b;
	}
	Rows=a1->cell;
	B=b1->elements;
	N=a1->rows;
	/*      */
	for(k=0;k<N;k++)
	{
		/*	        -
			   ,   k- 
			 ,     . */
		ind=k;
		max=v_abs(Rows[ind][k]);
		for(i=ind+1;i<N;i++)
		{
			elem=v_abs(Rows[i][k]);
			if(elem>max)
			{
				ind=i;
				max=elem;
			}
		}
		if(IsZero(max)) 
		{
			free(a1);
			free(b1);
			return NULL;
		}
		/*    A    B */
		if(ind!=k)
		{
			row=Rows[ind];
			Rows[ind]=Rows[k];
			Rows[k]=row;
			b=B[ind];
			B[ind]=B[k];
			B[k]=b;
		}
		/*  . */
		elem = Rows[k][k];
		for(j=k+1;j<N;j++)
			Rows[k][j]/=elem;
		B[k]/=elem;
		/*	      :
			- : (0,k+1) - (k-1,N-1)
			- :  (k+1,k+1) - (N-1,N-1)
			     .
		 */
		for(i=0;i<k;i++)
		{
			for(j=k+1;j<N;j++)
				Rows[i][j]-=Rows[i][k]*Rows[k][j];
			B[i]-=Rows[i][k]*B[k];
		}
		for(i=k+1;i<N;i++)
		{
			for(j=k+1;j<N;j++)
				Rows[i][j]-=Rows[i][k]*Rows[k][j];
			B[i]-=Rows[i][k]*B[k];
		}
	}
	free(a1);
	return b1;
}

/*
 * matInverse
 */
matrix_p V_GLOBAL
matInverse(matrix_t *m,vboolean keepon)
{
	u32_t i,j,k,ind;
	mvec_t elem,max,*row;
	mvec_t **RowsS;
	mvec_t **RowsD;
	u32_t N;
	matrix_t *temp;
	matrix_t *duple;
	V_ASSERT(m);
	V_ASSERT(m->rows==m->cols);
	temp=matAlloc(m->rows,m->cols);
	matUnit(temp);
	if(keepon)
		duple=matDuplicate(m);
	else
		duple=m;
	RowsS=duple->cell;
	RowsD=temp->cell;
	N=duple->rows;
	/*    */
	for(k=0;k<N;k++)
	{
		/*       . */
		ind=k;
		max=v_abs(RowsS[ind][k]);
		for(i=ind+1;i<N;i++)
		{
			elem=v_abs(RowsS[i][k]);
			if(elem>max)
			{
				ind=i;
				max=elem;
			}
		}
		if(IsZero(max)) 
		{
			free(duple);
			free(temp);
			return NULL;
		}
		/*    */
		if(ind!=k)
		{
			row=RowsS[ind];
			RowsS[ind]=RowsS[k];
			RowsS[k]=row;
			row=RowsD[ind];
			RowsD[ind]=RowsD[k];
			RowsD[k]=row;
		}
		/*  . */
		elem = RowsS[k][k];
		for(j=k+1;j<N;j++)
			RowsS[k][j]/=elem;
		for(j=0;j<N;j++)
			RowsD[k][j]/=elem;
		/*	      :
		 *		- : (0,k+1) - (k-1,N-1)
		 *		- :  (k+1,k+1) - (N-1,N-1)
		 *	     .
		 */
		for(i=0;i<k;i++)
		{
			for(j=k+1;j<N;j++)
				RowsS[i][j]-=RowsS[i][k]*RowsS[k][j];
			for(j=0;j<N;j++)
				RowsD[i][j]-=RowsS[i][k]*RowsD[k][j];
		}
		for(i=k+1;i<N;i++)
		{
			for(j=k+1;j<N;j++)
				RowsS[i][j]-=RowsS[i][k]*RowsS[k][j];
			for(j=0;j<N;j++)
				RowsD[i][j]-=RowsS[i][k]*RowsD[k][j];
		}
	}
	free(duple);
	duple=matDuplicate(temp);
	free(temp);
	return duple;
}

#ifdef	MATRIX_NO_MACRO

void	V_GLOBAL
matZero(matrix_t *m)
{
	V_ASSERT(m);
	v_zerodw(m->data,m->rows*m->cols*(sizeof(mvec_t)>>2));
}  

u32_t		V_GLOBAL
matRowCount(matrix_t *m)
{
	V_ASSERT(m);
	return m->rows;
}

u32_t		V_GLOBAL
matColCount(matrix_t *m)
{
	V_ASSERT(m);
	return m->cols;
}

#endif	/* MATRIX_NO_MACRO */

/*</pre></strong></body></html>*/
/*EOF*/
