/*<html><body bgcolor=#000080><font color=#C0C0B0><strong><pre>*/
#include	"../include/v_lib.h"
#include	"../source/v_mt.h"
#include 	"../source/v_os.h"

#ifdef	CHECKED_BUILD
#define	THREAD_MAGIC	0x74687264
#define	MUTEX_MAGIC		0x4D555445
V_LOCAL	char	*error_mutex_magic="bad mutex handle, magic wrong";
V_LOCAL	char	*error_thread_magic="bad thread handle, magic wrong";
#endif	/* CHECKED_BUILD */

V_LOCAL	void	*main_thread_tsd=NULL;

#if		defined(__WIN32__)

#include "win32/argsused.h"
V_LOCAL	LPVOID WINAPI
TlsGetValueStub(DWORD Key)
{
	return main_thread_tsd;
}

#include "win32/argsused.h"
V_LOCAL	BOOL WINAPI
TlsSetValueStub(DWORD Key,LPVOID Value)
{
	main_thread_tsd=Value;
	return TRUE;
}

LPVOID	(WINAPI *TlsGetValuePtr)(DWORD)=TlsGetValueStub;
BOOL	(WINAPI *TlsSetValuePtr)(DWORD,LPVOID)=TlsSetValueStub;

void	tsd_switch_to_system(void)
{
	TlsSetValuePtr=TlsSetValue;
	TlsGetValuePtr=TlsGetValue;
}

void	tsd_switch_to_stubs(void)
{
	TlsSetValuePtr=TlsSetValueStub;
	TlsGetValuePtr=TlsGetValueStub;
}

/*
 *	Thread object
 */
typedef struct	os_thread_s
{
#ifdef	CHECKED_BUILD
	u32_t			magic;
#endif	/* CHECKED_BUILD */
	HANDLE			handle;
	DWORD			id;
	void			*arg;
	thread_proc_t	start_address;
}	os_thread_t, *os_thread_p;

/*
 *	Mutex object
 */
typedef struct	os_mutex_s
{
#ifdef	CHECKED_BUILD
	u32_t		magic;
#endif	/* CHECKED_BUILD */
	HANDLE		mutex;
}	os_mutex_t, *os_mutex_p;

/*
 *	Thread entry point
 */
DWORD	CALLBACK
Win32ThreadEntry(LPVOID param)
{
	((os_thread_t *)param)->start_address(((os_thread_t *)param)->arg);
	threadExit(0);
	return 0;
}

V_LOCAL	DWORD	tsd_key;

#elif	defined(__UNIX__)

int
pthread_setspecific_stub(pthread_key_t key,const void *pointer)
{
	main_thread_tsd=(void*)pointer;
	return 0;
}


void*
pthread_getspecific_stub(pthread_key_t key)
{
	return main_thread_tsd;
}

int	(*pthread_setspecific_ptr)(pthread_key_t,const void *)=
	pthread_setspecific_stub;

void*	(*pthread_getspecific_ptr)(pthread_key_t)=
	pthread_getspecific_stub;

void	tsd_switch_to_system(void)
{
	pthread_setspecific_ptr=pthread_setspecific;
	pthread_getspecific_ptr=pthread_getspecific;
}

void	tsd_switch_to_stubs(void)
{
	pthread_setspecific_ptr=pthread_setspecific_stub;
	pthread_getspecific_ptr=pthread_getspecific_stub;
}

/*
 *	Thread object
 */
typedef struct	os_thread_s
{
#ifdef	CHECKED_BUILD
	u32_t			magic;
#endif	/* CHECKED_BUILD */
	pthread_t		id;
	pthread_attr_t	attr;
	void			*arg;
	thread_proc_t	start_address;
}	os_thread_t, *os_thread_p;

/*
 *	Mutex object
 */
typedef struct	os_mutex_s
{
#ifdef	CHECKED_BUILD
	u32_t			magic;
#endif	/* CHECKED_BUILD */
	pthread_mutex_t		handle;
	pthread_mutexattr_t	attr;
}	os_mutex_t, *os_mutex_p;

/*
 *	Thread entry point
 */
void*
PosixThreadEntry(void *param)
{
	((os_thread_t *)param)->start_address(((os_thread_t *)param)->arg);
	threadExit(0);
	return NULL;
}

V_LOCAL	pthread_key_t	tsd_key;

#endif	/* __WIN32__ */

/*	=======================================================================
**	TSD functions
**	TODO: move TLS key HERE from v_mt.c (it's almost already possible !)
*/

V_LOCAL	unsigned		tsd_key_count=0;

V_LOCAL	vboolean
tsd_create_key(void)
{
#ifdef	__WIN32__
	tsd_key=TlsAlloc();
	if(!tsd_key)
	{
		/* Can't use conlPrintf() ! */
		if(MessageBox(NULL,"tsd_per_thread_init(): "
			"TlsAlloc() failed, prepare for DrWatson() !",
			"Thread initialization error",
			MB_OKCANCEL|MB_ICONHAND)==IDOK)
			return false;
		else
			ExitProcess(1);
	}
#elif	defined(__UNIX__)
	if(pthread_key_create(&tsd_key,NULL))
	{
		/* Can't use conlPrintf() ! */
		int resp;
		printf("tsd_per_thread_init(): pthread_key_create() failed,"
			" prepare for core dump !\nContinue ? [Y|N] ");
		for(;;)
		{
			resp=fgetc(stdin);
			switch(resp)
			{
				case 'Y':
				case 'y':
					return false;
				case 'N':
				case 'n':
					exit(-1);
				default:
					puts("\nPlease, press Y or N: ");
			}
		}
	}
#endif	/* __WIN32__ */
	return true;
}

V_LOCAL void
tsd_destroy_key(void)
{
#ifdef	__WIN32__
	TlsFree(tsd_key);
#elif	defined(__UNIX__)
	pthread_key_delete(tsd_key);
#endif	/* __WIN32__ */
}

void	V_GLOBAL
threadSetSpecific(void_p specific)
{
#ifdef	__WIN32__
	TlsSetValuePtr(tsd_key,specific);
#elif	defined(__UNIX__)
	pthread_setspecific_ptr(tsd_key,specific);
#endif	/* __WIN32__ */
}

void_p		V_GLOBAL
threadGetSpecific(void)
{
#ifdef	__WIN32__
	return TlsGetValuePtr(tsd_key);
#elif	defined(__UNIX__)
	return pthread_getspecific_ptr(tsd_key);
#endif	/* __WIN32__ */
}

/*	=======================================================================
 *	Threads
 */
thread_t	V_GLOBAL
threadCreate(thread_proc_t start_address,void *arg)
{
#ifdef	__UNIX__
	struct sched_param sp;
#endif	/* __UNIX__ */
	os_thread_t *thread;
	if(!start_address)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"threadCreate(NULL,%X): no thread entry provided.\n",
			arg);
#endif	/* CHECKED_BUILD */
		return NULL;
	}
	thread=(os_thread_t*)malloc(sizeof(os_thread_t));
	thread->start_address=start_address;
	thread->arg=arg;
#ifdef	CHECKED_BUILD
	thread->magic=THREAD_MAGIC;
#endif	/* CHECKED_BUILD */
	/*	If this is first thread, initialize key, store local tsd
	 *	and switch TSD functions */
	tsd_key_count++;
	if(tsd_key_count==1)
	{
		void *main_specific=threadGetSpecific();
		tsd_create_key();
		tsd_switch_to_system();
		threadSetSpecific(main_specific);
	}
#ifdef	__WIN32__
	thread->handle=CreateThread(NULL,0,Win32ThreadEntry,thread,0,
		&thread->id);
	if(!thread->handle)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"threadCreate(%X,%X): CreateThread() "
			"failed.\n",start_address,arg);
#endif	/* CHECKED_BUILD */
		free(thread);
		return NULL;
	}
#elif	defined(__UNIX__)	/* !__WIN32__ */
	pthread_attr_init(&thread->attr);
	pthread_attr_setscope(&thread->attr,PTHREAD_SCOPE_PROCESS);
	pthread_attr_setdetachstate(&thread->attr,PTHREAD_CREATE_JOINABLE);
	if(!pthread_attr_getschedparam(&thread->attr,&sp))
	{
		sp.sched_priority=5;
		pthread_attr_setschedparam(&thread->attr,&sp);
	}
	if(pthread_create(&thread->id,&thread->attr,
		PosixThreadEntry,(void*)thread)==-1)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,"threadCreate(%X,%X): pthread_create() "
			"failed.\n",start_address,arg);
#endif	/* CHECKED_BUILD */
		free(thread);
		return NULL;
	}
#endif	/* !__UNIX__ */
	return (thread_t)thread;
}

void		V_GLOBAL
threadExit(u32_t exit_code)
{
/*
	void *buf;
	buf=threadGetSpecific();
	if(buf)
		free(buf);
*/
#ifdef	__WIN32__
	ExitThread(exit_code);
#elif	defined(__UNIX__)
	pthread_exit((void*)exit_code);
#endif	/* __WIN32__ */
}

vboolean	V_GLOBAL
threadJoin(thread_t thread,u32_t *exit_code)
{
	if(!thread)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"threadJoin(NULL,%X): invalid parameter.\n",
			exit_code);
#endif	/* CHECKED_BUILD */
		return false;
	}
#ifdef	CHECKED_BUILD
	if(((os_thread_t*)thread)->magic!=THREAD_MAGIC)
	{
		conlPrintf(LOG_ERROR,
			"threadJoin(): %s.\n",error_thread_magic);
		return false;
	}
#endif	/* CHECKED_BUILD */
#ifdef	__WIN32__
	if(WaitForSingleObject(((os_thread_t*)thread)->handle,INFINITE)
		!=WAIT_OBJECT_0)
		return false;
	if(exit_code)
	{
		DWORD ec;
		GetExitCodeThread(((os_thread_t*)thread)->handle,&ec);
		*exit_code=(u32_t)ec;
	}
#elif	defined(__UNIX__)
	if(pthread_join(((os_thread_t*)thread)->id,
		exit_code ? (void**)exit_code : NULL))
		return false;
#endif	/* __WIN32__ */
	/*	If this is the last thread, delete key, restore local tsd
	 *	and switch TSD functions back to stubs */
	/*	BUGBUG: Is this safe enought ??? */
	tsd_key_count--;
	if(tsd_key_count==1)
	{
		void *main_specific=threadGetSpecific();
		tsd_switch_to_stubs();
		tsd_destroy_key();
		threadSetSpecific(main_specific);
	}
	free(thread);
	return true;
}

/*	=======================================================================
 *	Mutexes
 */
mutex_t		V_GLOBAL
mutexCreate()
{
	os_mutex_t *mutex;
	mutex=(os_mutex_t*)malloc(sizeof(os_mutex_t));
#ifdef	CHECKED_BUILD
	mutex->magic=MUTEX_MAGIC;
#endif	/* !CHECKED_BUILD */
#ifdef	__WIN32__
	mutex->mutex=CreateMutex(NULL,FALSE,NULL);
#elif	defined(__UNIX__) /* !__WIN32__ */
	pthread_mutexattr_init(&mutex->attr);
#ifdef	__USE_UNIX98
	pthread_mutexattr_settype(&mutex->attr,PTHREAD_MUTEX_RECURSIVE_NP);
#endif	/* !__USE_UNIX98 */
#endif	/* !__UNIX__ */
	return mutex;
}

void	V_GLOBAL
mutexLock(mutex_t mutex)
{
	if(!mutex)
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"mutexLock(NULL): invalid parameter.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
#ifdef	CHECKED_BUILD
	if(((os_mutex_t *)mutex)->magic!=MUTEX_MAGIC)
	{
		conlPrintf(LOG_ERROR,
			"mutexLock(): %s.\n",error_mutex_magic);
		return;
	}
#endif	/* CHECKED_BUILD */
#ifdef	__WIN32__
	WaitForSingleObject(((os_mutex_t *)mutex)->mutex,INFINITE);
#elif	defined(__UNIX__) /* !__WIN32__ */
	pthread_mutex_lock(&((os_mutex_t*)mutex)->handle);
#endif	/* !__UNIX__ */
}

vboolean	V_GLOBAL
mutexTryLock(mutex_t mutex)
{
	/*	Calling mutexTryLock(NULL) is NOT considered as
	 *	an error. */
	if(!mutex)
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_INFO,
			"mutexTryLock(NULL): invalid handle, impossible to lock.\n");
#endif	/* CHECKED_BUILD */
		return false;
	}
#ifdef	CHECKED_BUILD
	if(((os_mutex_t *)mutex)->magic!=MUTEX_MAGIC)
	{
		conlPrintf(LOG_ERROR,
			"mutexTryLock(): %s.\n",error_mutex_magic);
		return false;
	}
#endif	/* CHECKED_BUILD */
#ifdef	__WIN32__
	if(WaitForSingleObject(((os_mutex_t *)mutex)->mutex,0)
		==WAIT_OBJECT_0)
		return true;
	else
		return false;
#elif	defined(__UNIX__)
	if(pthread_mutex_trylock(&((os_mutex_t *)mutex)->handle)!=0)
		return false;
	else
		return true;
#endif	/* __WIN32__ */
}

void	V_GLOBAL
mutexUnlock(mutex_t mutex)
{
	if(!mutex)
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"mutexUnlock(NULL): invalid parameter.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
#ifdef	CHECKED_BUILD
	if(((os_mutex_t *)mutex)->magic!=MUTEX_MAGIC)
	{
		conlPrintf(LOG_ERROR,
			"mutexUnlock(): %s.\n",error_mutex_magic);
		return;
	}
#endif	/* CHECKED_BUILD */
#ifdef	__WIN32__
	ReleaseMutex(((os_mutex_t *)mutex)->mutex);
#elif	defined(__UNIX__)
	pthread_mutex_unlock(&((os_mutex_t *)mutex)->handle);
#endif	/* __WIN32__ */
	return;
}

void		V_GLOBAL
mutexDestroy(mutex_t mutex)
{
	if(!mutex)
	{
#ifdef	CHECKED_BUILD
		conlPuts(LOG_ERROR,"mutexDestroy(NULL): invalid parameter.\n");
#endif	/* CHECKED_BUILD */
		return;
	}
#ifdef	CHECKED_BUILD
	if(((os_mutex_t *)mutex)->magic!=MUTEX_MAGIC)
	{
		conlPrintf(LOG_ERROR,
			"mutexDestroy(): %s.\n",error_mutex_magic);
		return;
	}
#endif	/* CHECKED_BUILD */
#ifdef	__UNIX__
	pthread_mutexattr_destroy(&((os_mutex_t*)mutex)->attr);
#endif	/* __UNIX__ */
	free(mutex);
}

/*</pre></strong></body></html>*/
/*EOF*/
