/*<html><body bgcolor=#000080><font color=#C0C0B0><strong><pre>*/
#include	"../include/v_lib.h"
#include	"../source/v_mt.h"

/*	we declare defstr quite large because default text 
 *	'[NULL]' can later be replaced by:
 *	1. Explicit call: strcpy(dbs(NULL),"New text");
 *	2. Implicit: due to caller bugs.
 */
static char defstr[TRASHED_BUFFER_SIZE];

char_p	V_GLOBAL
dbs(char_p string_which_can_be_NULL)
{
#ifdef	CHECKED_BUILD
	if(string_which_can_be_NULL)
		return string_which_can_be_NULL;
	else
	{
		/* Avoid infinite recursion if TRASHED_BUFFER_SIZE==0 */
		V_ASSERT(TRASHED_BUFFER_SIZE);
		strCopy(defstr,"[NULL]",TRASHED_BUFFER_SIZE);
		return defstr;
	}
#else
	return string_which_can_be_NULL;
#endif	/* CHECKED_BUILD */
}

/*
 *	strCopy - analog to strncpy.
 *	Differences:
 *	1. Remember: third parameter is AMOUNT OF MEMORY allocated for 'dst'.
 *	2. 'Dst' is ALWAYS terminated by null-character ('\0').
 *	3. Strings 'dst' and 'src' can be overlapped.
 *	4. Ifdef CHECHED_BUILD and dst==NULL, default static buffer returned.
 */
char_p V_GLOBAL
strCopy(char_p dst,char_p src,int dest_storage_length)
{
	int len;
	if(!dst || !src || dest_storage_length<=0)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"strCopy('%s','%s',%d): incorrect parameters.\n",dbs(dst),
			dbs(src),dest_storage_length);
#endif	/* CHECKED_BUILD */
		if(dst && dest_storage_length>0)
			dst[0]=0;
		return dbs(dst);
	}
	len=strlen(src);
	if(len>=dest_storage_length)
		len=dest_storage_length-1;
	memmove(dst,src,len);
	dst[len]=0;
	return dst;
}

/*
 *	strCat - analog to strncat.
 *	Differences:
 *	1. Strings 'dst' and 'src' can be overlapped.
 *	2. Ifdef CHECHED_BUILD and dst==NULL, default static buffer returned.
 */
char_p	V_GLOBAL
strCat(char_p dst,char_p src,int max_chars_to_be_added)
{
	if(!dst || !src || max_chars_to_be_added<=0)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"strCat('%s','%s',%d): incorrect parameters.\n",dbs(dst),
			dbs(src),max_chars_to_be_added);
#endif	/* CHECKED_BUILD */
		return dbs(dst);
	}
	return strCopy(dst+strlen(dst),src,max_chars_to_be_added);
}

/*
 *	Strips leading whitespace from string. Returns str.
 *	Note. Ifdef CHECHED_BUILD and str==NULL, default static buffer returned.
 */
char_p	V_GLOBAL
strStripLead(char_p str)
{
	char *ptr;
	if(!str)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"strStripLead('%s'): incorrect parameter.\n",dbs(str));
#endif	/* CHECKED_BUILD */
		return dbs(str);
	}
	ptr=str;
	while(*ptr && *ptr<' ')
		ptr++;
	if(ptr != str)
		memmove(str,ptr,strlen(ptr)+1);
	return str;
}

/*
 *	Strips trailing whitespace from string. Returns str.
 *	Note. Ifdef CHECHED_BUILD and str==NULL, default static buffer returned.
 */
char_p	V_GLOBAL
strStripTrail(char_p str)
{
	char *ptr;
	if(!str)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"strStripTrail('%s'): incorrect parameter.\n",dbs(str));
#endif	/* CHECKED_BUILD */
		return dbs(str);
	}
	ptr=str+strlen(str)-1;
	for(;ptr>str && *ptr<' ';ptr--)
		*ptr=0;
	return str;
}

/*
 *	Strips both leading and trailing whitespace. Returns str.
 *	Note. Ifdef CHECHED_BUILD and str==NULL, default static buffer returned.
 */
char_p	V_GLOBAL
strStripLT(char_p str)
{
	if(!str)
	{
#ifdef	CHECKED_BUILD
		conlPrintf(LOG_ERROR,
			"strStripLT('%s'): incorrect parameter.\n",dbs(str));
#endif	/* CHECKED_BUILD */
		return dbs(str);
	}
	strStripLead(str);
	strStripTrail(str);
	return str;
}

/*</pre></strong></body></html>*/
/*EOF*/
